@@ -97,7 +97,7 @@ sen_bm_preBmBc(const unsigned char *x, size_t m, size_t * bmBc)
       cond->last_found = found; \
     } \
     cond->start_offset = offset; \
-    for (i = cond->last_found; i < cond->last_found + m; i++) { \
+    for (i = cond->last_found; i < found + m; i++) { \
       if (object->checks[i] > 0) { \
         offset += object->checks[i]; \
       } \
@@ -318,8 +318,7 @@ sen_snip_find_firstbyte(const char *string, sen_encoding encoding, size_t offset
     while (string[offset] <= (char) 0xc0)
       offset += doffset;
     break;
-  case sen_enc_default:
-  case sen_enc_none:
+  default:
     break;
   }
   return offset;
