@@ -912,6 +912,238 @@ normalize_none(sen_nstr *nstr)
   return sen_success;
 }
 
+/* use cp1252 as latin1 */
+inline static sen_rc
+normalize_latin1(sen_nstr *nstr)
+{
+  int_least8_t *ch;
+  const unsigned char *s, *s_;
+  unsigned char *d, *d0, *d_;
+  uint_least8_t *cp, *ctypes, ctype;
+  size_t size = strlen(nstr->orig), length = 0;
+  int removeblankp = nstr->flags & SEN_STR_REMOVEBLANK;
+  if (!(nstr->norm = SEN_MALLOC(size + 1))) {
+    return sen_memory_exhausted;
+  }
+  d0 = (unsigned char *) nstr->norm;
+  if (nstr->flags & SEN_STR_WITH_CHECKS) {
+    if (!(nstr->checks = SEN_MALLOC(size + 1))) {
+      return sen_memory_exhausted;
+    }
+  }
+  ch = nstr->checks;
+  if (nstr->flags & SEN_STR_WITH_CTYPES) {
+    if (!(nstr->ctypes = SEN_MALLOC(size + 1))) {
+      return sen_memory_exhausted;
+    }
+  }
+  cp = ctypes = nstr->ctypes;
+  for (s = s_ = (unsigned char *) nstr->orig, d = d_ = d0; *s; s++) {
+    unsigned char c = *s;
+    switch (c >> 4) {
+    case 0 :
+    case 1 :
+      /* skip unprintable ascii */
+      if (cp > ctypes) { *(cp - 1) |= SEN_NSTR_BLANK; }
+      continue;
+    case 2 :
+      if (c == 0x20) {
+	if (removeblankp) {
+	  if (cp > ctypes) { *(cp - 1) |= SEN_NSTR_BLANK; }
+	  continue;
+	} else {
+	  *d = ' ';
+	  ctype = SEN_NSTR_BLANK|sen_str_symbol;
+	}
+      } else {
+	*d = c;
+	ctype = sen_str_symbol;
+      }
+      break;
+    case 3 :
+      *d = c;
+      ctype = (c <= 0x39) ? sen_str_digit : sen_str_symbol;
+      break;
+    case 4 :
+      *d = ('A' <= c) ? c + 0x20 : c;
+      ctype = (c == 0x40) ? sen_str_symbol : sen_str_alpha;
+      break;
+    case 5 :
+      *d = (c <= 'Z') ? c + 0x20 : c;
+      ctype = (c <= 0x5a) ? sen_str_alpha : sen_str_symbol;
+      break;
+    case 6 :
+      *d = c;
+      ctype = (c == 0x60) ? sen_str_symbol : sen_str_alpha;
+      break;
+    case 7 :
+      *d = c;
+      ctype = (c <= 0x7a) ? sen_str_alpha : (c == 0x7f ? sen_str_others : sen_str_symbol);
+      break;
+    case 8 :
+      if (c == 0x8a || c == 0x8c || c == 0x8e) {
+	*d = c + 0x10;
+	ctype = sen_str_alpha;
+      } else {
+	*d = c;
+	ctype = sen_str_symbol;
+      }
+      break;
+    case 9 :
+      if (c == 0x9a || c == 0x9c || c == 0x9e || c == 0x9f) {
+	*d = (c == 0x9f) ? c + 0x60 : c;
+	ctype = sen_str_alpha;
+      } else {
+	*d = c;
+	ctype = sen_str_symbol;
+      }
+      break;
+    case 0x0c :
+      *d = c + 0x20;
+      ctype = sen_str_alpha;
+      break;
+    case 0x0d :
+      *d = (c == 0xd7 || c == 0xdf) ? c : c + 0x20;
+      ctype = (c == 0xd7) ? sen_str_symbol : sen_str_alpha;
+      break;
+    case 0x0e :
+      *d = c;
+      ctype = sen_str_alpha;
+      break;
+    case 0x0f :
+      *d = c;
+      ctype = (c == 0xf7) ? sen_str_symbol : sen_str_alpha;
+      break;
+    default :
+      *d = c;
+      ctype = sen_str_others;
+      break;
+    }
+    d++;
+    length++;
+    if (cp) { *cp++ = ctype; }
+    if (ch) {
+      *ch++ = (int_least8_t)(s + 1 - s_);
+      s_ = s + 1;
+      while (++d_ < d) { *ch++ = 0; }
+    }
+  }
+  if (cp) { *cp = sen_str_null; }
+  *d = '\0';
+  nstr->length = length;
+  return sen_success;
+}
+
+inline static sen_rc
+normalize_koi8r(sen_nstr *nstr)
+{
+  int_least8_t *ch;
+  const unsigned char *s, *s_;
+  unsigned char *d, *d0, *d_;
+  uint_least8_t *cp, *ctypes, ctype;
+  size_t size = strlen(nstr->orig), length = 0;
+  int removeblankp = nstr->flags & SEN_STR_REMOVEBLANK;
+  if (!(nstr->norm = SEN_MALLOC(size + 1))) {
+    return sen_memory_exhausted;
+  }
+  d0 = (unsigned char *) nstr->norm;
+  if (nstr->flags & SEN_STR_WITH_CHECKS) {
+    if (!(nstr->checks = SEN_MALLOC(size + 1))) {
+      return sen_memory_exhausted;
+    }
+  }
+  ch = nstr->checks;
+  if (nstr->flags & SEN_STR_WITH_CTYPES) {
+    if (!(nstr->ctypes = SEN_MALLOC(size + 1))) {
+      return sen_memory_exhausted;
+    }
+  }
+  cp = ctypes = nstr->ctypes;
+  for (s = s_ = (unsigned char *) nstr->orig, d = d_ = d0; *s; s++) {
+    unsigned char c = *s;
+    switch (c >> 4) {
+    case 0 :
+    case 1 :
+      /* skip unprintable ascii */
+      if (cp > ctypes) { *(cp - 1) |= SEN_NSTR_BLANK; }
+      continue;
+    case 2 :
+      if (c == 0x20) {
+	if (removeblankp) {
+	  if (cp > ctypes) { *(cp - 1) |= SEN_NSTR_BLANK; }
+	  continue;
+	} else {
+	  *d = ' ';
+	  ctype = SEN_NSTR_BLANK|sen_str_symbol;
+	}
+      } else {
+	*d = c;
+	ctype = sen_str_symbol;
+      }
+      break;
+    case 3 :
+      *d = c;
+      ctype = (c <= 0x39) ? sen_str_digit : sen_str_symbol;
+      break;
+    case 4 :
+      *d = ('A' <= c) ? c + 0x20 : c;
+      ctype = (c == 0x40) ? sen_str_symbol : sen_str_alpha;
+      break;
+    case 5 :
+      *d = (c <= 'Z') ? c + 0x20 : c;
+      ctype = (c <= 0x5a) ? sen_str_alpha : sen_str_symbol;
+      break;
+    case 6 :
+      *d = c;
+      ctype = (c == 0x60) ? sen_str_symbol : sen_str_alpha;
+      break;
+    case 7 :
+      *d = c;
+      ctype = (c <= 0x7a) ? sen_str_alpha : (c == 0x7f ? sen_str_others : sen_str_symbol);
+      break;
+    case 0x0a :
+      *d = c;
+      ctype = (c == 0xa3) ? sen_str_alpha : sen_str_others;
+      break;
+    case 0x0b :
+      if (c == 0xb3) {
+	*d = c - 0x10;
+	ctype = sen_str_alpha;
+      } else {
+	*d = c;
+	ctype = sen_str_others;
+      }
+      break;
+    case 0x0c :
+    case 0x0d :
+      *d = c;
+      ctype = sen_str_alpha;
+      break;
+    case 0x0e :
+    case 0x0f :
+      *d = c - 0x20;
+      ctype = sen_str_alpha;
+      break;
+    default :
+      *d = c;
+      ctype = sen_str_others;
+      break;
+    }
+    d++;
+    length++;
+    if (cp) { *cp++ = ctype; }
+    if (ch) {
+      *ch++ = (int_least8_t)(s + 1 - s_);
+      s_ = s + 1;
+      while (++d_ < d) { *ch++ = 0; }
+    }
+  }
+  if (cp) { *cp = sen_str_null; }
+  *d = '\0';
+  nstr->length = length;
+  return sen_success;
+}
+
 sen_nstr *
 sen_nstr_open(const char *str, sen_encoding encoding, int flags)
 {
@@ -938,6 +1170,12 @@ sen_nstr_open(const char *str, sen_encoding encoding, int flags)
   case sen_enc_sjis :
     rc = normalize_sjis(nstr);
     break;
+  case sen_enc_latin1 :
+    rc = normalize_latin1(nstr);
+    break;
+  case sen_enc_koi8r :
+    rc = normalize_koi8r(nstr);
+    break;
   default :
     rc = normalize_none(nstr);
     break;
@@ -965,7 +1203,9 @@ const char * sen_enc_string[] = {
   "none",
   "euc_jp",
   "utf8",
-  "sjis"
+  "sjis",
+  "latin1",
+  "koi8r"
 };
 
 size_t
