@@ -125,17 +125,27 @@ segment_new(sen_sym *sym, int segtype)
   return sen_success;
 }
 
+#define SEG_LOAD(io,si)\
+{\
+  int32_t segno = si->segno;\
+  if (0 <= segno && segno < SEN_SYM_MAX_SEGMENT) {\
+    SEN_IO_SEG_REF(io, segno, si->addr);\
+    SEN_IO_SEG_UNREF(io, segno);\
+  }\
+}
+
 /* patricia array operation */
 
 inline static pat_node *
 pat_at(sen_sym *sym, sen_id id)
 {
-  sen_io_seginfo *si;
+  sen_sym_seginfo *si;
   if (id > SEN_SYM_MAX_ID) { return NULL; }
   si = &sym->patarray[id >> W_OF_PAT_IN_A_SEGMENT];
   if (!si->addr) {
     if (si->segno == -1) { load_all_segments(sym); }
-    if (sen_io_seg_bind(sym->io, si)) { return NULL; }
+    SEG_LOAD(sym->io, si);
+    if (!si->addr) { return NULL; }
   }
   return
     (pat_node *)(((byte *)si->addr) + ((id & PAT_MASK_IN_A_SEGMENT) * sizeof(pat_node)));
@@ -144,14 +154,15 @@ pat_at(sen_sym *sym, sen_id id)
 inline static pat_node *
 pat_get(sen_sym *sym, sen_id id)
 {
-  sen_io_seginfo *si;
+  sen_sym_seginfo *si;
   if (id > SEN_SYM_MAX_ID) { return NULL; }
   si = &sym->patarray[id >> W_OF_PAT_IN_A_SEGMENT];
   if (!si->addr) {
     while (si->segno == -1) {
       if (segment_new(sym, segment_pat)) { return NULL; }
     }
-    if (sen_io_seg_bind(sym->io, si)) { return NULL; }
+    SEG_LOAD(sym->io, si);
+    if (!si->addr) { return NULL; }
   }
   return
     (pat_node *)(((byte *)si->addr) + ((id & PAT_MASK_IN_A_SEGMENT) * sizeof(pat_node)));
@@ -176,12 +187,13 @@ pat_node_new(sen_sym *sym, sen_id *id)
 inline static sis_node *
 sis_at(sen_sym *sym, sen_id id)
 {
-  sen_io_seginfo *si;
+  sen_sym_seginfo *si;
   if (id > SEN_SYM_MAX_ID) { return NULL; }
   si = &sym->sisarray[id >> W_OF_SIS_IN_A_SEGMENT];
   if (!si->addr) {
     if (si->segno == -1) { load_all_segments(sym); }
-    if (sen_io_seg_bind(sym->io, si)) { return NULL; }
+    SEG_LOAD(sym->io, si);
+    if (!si->addr) { return NULL; }
   }
   return
     (sis_node *)(((byte *)si->addr) + ((id & SIS_MASK_IN_A_SEGMENT) * sizeof(sis_node)));
@@ -190,14 +202,15 @@ sis_at(sen_sym *sym, sen_id id)
 inline static sis_node *
 sis_get(sen_sym *sym, sen_id id)
 {
-  sen_io_seginfo *si;
+  sen_sym_seginfo *si;
   if (id == SEN_SYM_NIL || SEN_SYM_MAX_ID < id) { return NULL; }
   si = &sym->sisarray[id >> W_OF_SIS_IN_A_SEGMENT];
   if (!si->addr) {
     while (si->segno == -1) {
       if (segment_new(sym, segment_sis)) { return NULL; }
     }
-    if (sen_io_seg_bind(sym->io, si)) { return NULL; }
+    SEG_LOAD(sym->io, si);
+    if (!si->addr) { return NULL; }
   }
   return
     (sis_node *)(((byte *)si->addr) + ((id & SIS_MASK_IN_A_SEGMENT) * sizeof(sis_node)));
@@ -227,11 +240,12 @@ sis_collect(sen_sym *sym, sen_set *h, sen_id id, uint32_t level)
 inline static char *
 key_at(sen_sym *sym, uint32_t pos)
 {
-  sen_io_seginfo *si;
+  sen_sym_seginfo *si;
   si = &sym->keyarray[pos >> W_OF_KEY_IN_A_SEGMENT];
   if (!si->addr) {
     if (si->segno == -1) { load_all_segments(sym); }
-    if (sen_io_seg_bind(sym->io, si)) { return NULL; }
+    SEG_LOAD(sym->io, si);
+    if (!si->addr) { return NULL; }
   }
   return (char *)(((byte *)si->addr) + (pos & KEY_MASK_IN_A_SEGMENT));
 }
@@ -432,6 +446,7 @@ sen_sym_create(const char *path, uint32_t key_size,
   header->curr_del = 0;
   header->curr_del2 = 0;
   header->curr_del3 = 0;
+  header->lock = 0;
   for (i = 0; i < SEN_SYM_MAX_SEGMENT; i++) {
     header->segments[i] = segment_empty;
   }
